#include "ApplicationList.h"
#include <QWidget>
#include <QVariant>
#include <QFrame>


ApplicationList::ApplicationList (MaemoApplicationHandler *handler, QWidget *parent): QWidget  (parent) {
    busy=false;
    load = true;
    appHandler = handler;
    appListWidgets = new QList <ApplicationListWidget*> ();
    main = parent;
    this->setFixedSize (800,426);
    unreceivedPreviews = 0;
    connect (appHandler, SIGNAL(applicationListReady(QList<MaemoApplication*>*)),this,SLOT(listDelivered(QList<MaemoApplication*>*)));

    loading = new QLabel ("loading",this);
    loading->setStyleSheet("color: white;background-color:black");
    loading->setGeometry (0,0,800,372);

    //statusbar
    statusBar = new QWidget (this);
//    searchFrame = new QFrame (statusBar);
//    searchFrame->setGeometry (280,5,360,45);
//    searchFrame->setStyleSheet ("background-color: white;background-image: url("");");
//    searchLineEdit = new QLineEdit (searchFrame);
//    searchLineEdit->setFixedSize(360,45);
//    searchLineEdit->setFrame(false);
//    searchButton = new QPushButton (statusBar);
//    searchButton->setText("Search");
//    searchButton->setGeometry (640,5,150,45);
//    searchButton->setFlat (true);
//    searchButton->setStyleSheet (":flat{background-image: url(:/images/background/install_bar.png); border: none;}");
//    connect (searchButton,SIGNAL(clicked()),this,SLOT(search()));


    statusBar->setStyleSheet ("background-image: url(:/images/background/bar.png)");
    statusBar->setGeometry (0,372,800,54);
}

ApplicationList::~ApplicationList () {
    int a;
    for (a = 0; a < appListWidgets -> length (); a++) {
          delete appListWidgets->at(a);
    }
    if (!loading) {
        delete this->appList;
    }
    delete statusBar;
    //delete this->appListWidgets;
}

void ApplicationList::requestApplications (int listType,QString OS,QString cat, int page, int pageSize, QString search,QString sort) {
    loading = new QLabel();
    if (busy) {
        // notification
    } else {
        QTextStream out (stdout);
        busy = true;
        type = listType;
        appHandler->requestApplications (OS,cat,page,pageSize,search,sort);
    }
}


/* slot when the list is delivered
 * show list of available applications in window
 * @todo: limit -> next, prev page */
void ApplicationList::listDelivered(QList <MaemoApplication *> *maemoAppList) {
    // delete old list if it existed
    int a;
    if (appListWidgets->length ()>0) {
        for (a = 0; a < appListWidgets -> length (); a++) {
            delete appListWidgets->at(a);
        }
        appListWidgets = new QList <ApplicationListWidget*> ();
        delete appList;
        appList = new QListWidget (this);
        appList->setGeometry (0,0,800,372);
    } else {
        appList = new QListWidget (this);
        appList->setGeometry (0,0,800,372);
    }
    load = false;
    appList->setHorizontalScrollBarPolicy (Qt::ScrollBarAlwaysOff);
    QTextStream out (stdout);
    delete loading;
    ApplicationListWidget *app;
    QListWidgetItem *item;
    int length = maemoAppList->length();
    connect (appHandler, SIGNAL( imageReady (QPixmap*,QString,QString)),this,SLOT(previewImageDelivered(QPixmap*,QString,QString)));
    for (a = 0; a < length; a++) {
        app = new ApplicationListWidget(maemoAppList->at(a));
        //connect (app, SIGNAL(clicked(bool,MaemoApplication*)),this,SLOT(appButtonClicked(bool,MaemoApplication*)));
        out << maemoAppList->at(a)->toString() << endl;
        appListWidgets -> append (app);
        item = new QListWidgetItem();
        item->setSizeHint(QSize(800,120));
        item->setData (APPID,QVariant(maemoAppList->at(a)->getUid()));
        item->setData (APPWIDGET,QVariant(appListWidgets->indexOf(app)));
        appList->addItem(item);
        appList->setItemWidget (item, app);
        if (!(maemoAppList->at(a)->getPreview()).isEmpty()) {
            out << maemoAppList->at(a)->getPreview()<<endl;
            //appHandler->requestImage(maemoAppList->at(a)->getPreview(),QString::number(a));
            unreceivedPreviews ++;
        }
        else {
            out << "without: "+maemoAppList->at(a)->getTitle()<<endl;
        }
    }
    if (unreceivedPreviews==0) {
        disconnect (appHandler, SIGNAL( imageReady (QPixmap*,QString,QString)),this,SLOT(previewImageDelivered(QPixmap*,QString,QString)));
    }
    connect (appList, SIGNAL(itemClicked(QListWidgetItem *)),this,SLOT(appButtonClicked(QListWidgetItem*)));
    connect (appList, SIGNAL(itemPressed(QListWidgetItem*)),this,SLOT(appButtonPressed(QListWidgetItem*)));
    appList->show();
    for (a=0;a<maemoAppList->length();a++) {
        appHandler->requestImage(maemoAppList->at(a)->getPreview(),QString::number(a),"",true);
    }
    busy = false;

}

void ApplicationList::previewImageDelivered(QPixmap * pre, QString id, QString) {
    QTextStream out (stdout);
    appListWidgets->at(id.toInt())->setPreview(new QPixmap(pre->scaled(195,117)));
    unreceivedPreviews --;
    if (unreceivedPreviews==0) {
        disconnect (appHandler, SIGNAL( imageReady (QPixmap*,QString,QString)),this,SLOT(previewImageDelivered(QPixmap*,QString,QString)));
    }
}

//open window with application details
void ApplicationList::singleAppDelivered(MaemoApplicationDetails *app) {
    disconnect (appHandler,SIGNAL(applicationDetailsReady(MaemoApplicationDetails*)),this,SLOT(singleAppDelivered(MaemoApplicationDetails*)));
    ApplicationDetailsWidget *appDetails = new ApplicationDetailsWidget (app,appHandler,main);
    appDetails->setGeometry (0,54,800,426);
    connect (appDetails,SIGNAL(layerAdded(QWidget*,QString,QString)),this,SLOT(widgetLayerAdded(QWidget*,QString,QString)));
    emit layerAdded(appDetails,app->getTitle(),"");
    appDetails->show();
    appListWidgets->at (appList->currentItem()->data(APPWIDGET).toInt())->unselect();
}

void ApplicationList::appButtonPressed(QListWidgetItem *item) {
    appListWidgets->at(item->data(APPWIDGET).toInt())->select();
}

void ApplicationList::widgetLayerAdded(QWidget * w,QString bc1,QString bc2) {
    emit layerAdded(w,bc1,bc2);
}


//application button is clicked
//-> request details
void ApplicationList::appButtonClicked (QListWidgetItem *item) {
    appList->setCurrentItem (item);
    appHandler->requestApplicationData(item->data(APPID).toString());
    //appListWidgets->at(item->data(APPWIDGET).toInt())->unselect();
    connect (appHandler,SIGNAL(applicationDetailsReady(MaemoApplicationDetails*)),this,SLOT(singleAppDelivered(MaemoApplicationDetails*)));
}

void ApplicationList::search () {
    QString s = searchLineEdit->text();
    requestApplications (ApplicationList::ALL,"Maemo5","",0,999,s);
}

void ApplicationList::refetchData(bool) {
    //appHandler->requestAllApplications();
}

// is called, when a connection couldn't be established
// updates the window with this information
void ApplicationList::connectionError() {
}


