#include "mainwindow.h"
#include "ui_mainwindow.h"
#include <QScrollArea>

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    m_ui(new Ui::MainWindow)
{
    m_ui->setupUi(this);

    handler = new MaemoApplicationHandler();

//    applicationList = new ApplicationList (m_ui->centralwidget,m_ui->mainWidget);
//    applicationList -> requestApplications (ApplicationList::ALL,"Maemo5","",0,10,"","down");
    categoryList = new CategoryListWidget (m_ui->mainWidget);
    connect (categoryList, SIGNAL (itemClicked(QListWidgetItem*)),this, SLOT(openApplicationsList(QListWidgetItem*)));
    //scroll->setWidget (applicationList);
    //scroll->setGeometry (0,0,800,370);
    topWidgets = new QList <SubWindow*>();

    //*topBarWidget*//
    m_ui->topBarWidget->setStyleSheet ("background-image: url(:/images/background/bar.png);");

    //button settings of topBar
    m_ui->closeButton->setStyleSheet (":flat{image: url(:/images/icons/cross.png);background-color: none; border: none;}");

    //settings of topBar-labels
    m_ui->breadcrumb2Label->setStyleSheet ("color: white");
    m_ui->breadcrumb1Label->setStyleSheet ("color: white");
    m_ui->logoButton->setStyleSheet ("background-image: url(:/images/icons/maemo.png);border:none;");
    m_ui->firstSeparatorLabel->setStyleSheet ("background-image: url(:/images/icons/bar_arrow.png);border:none;background-repeat: no-repeat");
    //->setPixmap (QString ("url(:/images/icons/bar_arrow.png)"));
    //m_ui->secondSeparatorLabel->setPixmap (QString ("images/icons/bar_arrow.png"));
    m_ui->secondSeparatorLabel->setStyleSheet ("background-image: url(:/images/icons/bar_arrow.png);border:none;background-repeat: no-repeat");

    //*statusBarWidget*//
    m_ui->statusBarWidget->setStyleSheet ("background-image: url(:/images/background/bar.png);");

    //buttons
    m_ui->searchButton->setStyleSheet (":flat{color: white; background-color: none; border: none; background-image: url(:/images/background/install_bar.png);}");
    m_ui->backButton->setVisible(false);
    m_ui->backButton->setStyleSheet (":flat{image: url(:/images/icons/back_arrow.png);background-color: none; border: none;}");

    m_ui->keywordLineEdit->setFrame(false);
    m_ui->searchFrame->setStyleSheet ("background-color: white;background-image: url("");");

    DEFAULTBC1="Maemo 5";
    DEFAULTBC2="";
    setBreadCrumbs("","",true);

}


MainWindow::~MainWindow()
{
    delete m_ui;
}

void MainWindow::openApplicationsList(QListWidgetItem *item) {
    ApplicationList *applicationList = new ApplicationList (handler,m_ui->centralwidget);
    applicationList -> requestApplications (ApplicationList::ALL,
                                            item->data(CategoryListWidget::MAEMOVERSION).toString(),
                                            item->data(CategoryListWidget::CATEGORY).toString(),
                                            item->data(CategoryListWidget::PAGE).toInt(),
                                            item->data(CategoryListWidget::PAGESIZE).toInt(),
                                            "",
                                            item->data(CategoryListWidget::SORTING).toString());
    applicationList->setGeometry(0,54,800,426);
    connect (applicationList, SIGNAL (layerAdded (QWidget*,QString,QString)),this, SLOT(addWidgetLayer(QWidget*,QString,QString)));
    applicationList->show();
    addWidgetLayer (applicationList,"Maemo 5",item->data(CategoryListWidget::TITLE).toString());
}


void MainWindow::sendSearchRequest () {
    QString k = m_ui->keywordLineEdit->text();
    ApplicationList *applicationList = new ApplicationList (handler,m_ui->centralwidget);
    applicationList->requestApplications (ApplicationList::ALL,"Maemo5","",0,999,k);
    addWidgetLayer (applicationList,"Maemo 5","Search: "+k);
    applicationList->setGeometry(0,54,800,426);
    connect (applicationList, SIGNAL (layerAdded (QWidget*,QString,QString)),this, SLOT(addWidgetLayer(QWidget*,QString,QString)));
    applicationList->show();
}

void MainWindow::goHome () {
    //applicationList->requestApplications (ApplicationList::HOTTEST,"Maemo5","",0,10,"","down");
}

void MainWindow::addWidgetLayer (QWidget* w, QString bc1, QString bc2) {
    if (topWidgets->length()==0) {
        m_ui->backButton->setVisible(true);
    }
    topWidgets->append(new SubWindow (w,bc1,bc2));
    setBreadCrumbs(bc1,bc2);
}

void MainWindow::removeUpperLayer() {
    if (topWidgets->length()>0) {
        delete topWidgets->takeLast();
    }
    if (topWidgets->length()==0) {
        m_ui->backButton->setVisible(false);
        setBreadCrumbs("","",true);
    }
    else {
        setBreadCrumbs(topWidgets->last()->getBreadcrumb_1(),topWidgets->last()->getBreadcrumb_2());
    }
}

void MainWindow::setBreadCrumbs (QString bc1, QString bc2, bool setDefault) {

    if (setDefault) {
        bc1= DEFAULTBC1;
        bc2= DEFAULTBC2;
    }
    m_ui->breadcrumb1Label->setText(bc1);
    int width = m_ui->breadcrumb1Label->fontMetrics().width(bc1);
    m_ui->breadcrumb1Label->setFixedSize (width,54);
    if (bc2.isEmpty()) {
        m_ui->secondSeparatorLabel->setVisible (false);
    } else {
        m_ui->secondSeparatorLabel->setVisible (true);
        m_ui->secondSeparatorLabel->setGeometry (150+width,0,20,54);
    }
    m_ui->breadcrumb2Label->setText(bc2);
    m_ui->breadcrumb2Label->setFixedSize(m_ui->breadcrumb1Label->fontMetrics().width(bc2),54);
    m_ui->breadcrumb2Label->setGeometry (190+width,0,20,54);
}

void MainWindow::goBackToList () {
    int k;
    for (k=0;k<topWidgets->length();k++) {
        delete topWidgets->at(k);
    }
    delete topWidgets;
    topWidgets = new QList <SubWindow*>();
}

